<?php

namespace Drupal\disclosure_menu\Plugin\Block;

use Drupal\system\Plugin\Block\SystemMenuBlock;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuActiveTrailInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a "Disclosure Menu" block.
 *
 * @Block(
 *   id = "disclosure_menu",
 *   admin_label = @Translation("Disclosure menu"),
 *   cache = -1,
 *   category = @Translation("Disclosure Menu"),
 *   deriver = "Drupal\system\Plugin\Derivative\SystemMenuBlock"
 * )
 */
class DisclosureMenuBlock extends SystemMenuBlock {

  /**
   * The active menu trail service.
   *
   * @var \Drupal\Core\Menu\MenuActiveTrailInterface
   */
  protected $menuActiveTrail;

  /**
   * Constructs a new SuperfishBlock.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_tree
   *   The menu tree service.
   * @param \Drupal\Core\Menu\MenuActiveTrailInterface $menu_active_trail
   *   The active menu trail service.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, MenuLinkTreeInterface $menu_tree, MenuActiveTrailInterface $menu_active_trail) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $menu_tree, $menu_active_trail);
    $this->menuActiveTrail = $menu_active_trail;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('menu.link_tree'),
      $container->get('menu.active_trail')
    );
  }

  /**
   * Overrides \Drupal\block\BlockBase::defaultConfiguration().
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'submenu_disclosure_levels' => 1,
      'include_default_js' => 1,
      'include_hover_js' => 0,
      'hover_show_delay' => 150,
      'hover_hide_delay' => 250,
    ];
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockForm().
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $form['submenu_disclosure_levels'] = [
      '#title' => $this->t('Maximum disclosure levels'),
      '#type' => 'select',
      '#options' => [
        0 => $this->t('None'),
        1 => 1,
        2 => 2,
        3 => 3,
        4 => 4,
        5 => 5,
        6 => 6,
        7 => 7,
        8 => 8,
        9 => 9,
      ],
      '#description' => $this->t('How many menu levels should include disclosure buttons for their submenus?'),
      '#default_value' => $this->configuration['submenu_disclosure_levels'],
    ];

    $form['include_default_js'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include default Javascript'),
      '#description' =>
      $this->t('This will toggle open submenus when clicking their disclosure button.'),
      '#default_value' => $this->configuration['include_default_js'],
      '#options' => [
        1 => 'Yes',
        0 => 'No',
      ],
    ];

    $form['include_hover_js'] = [
      '#type' => 'radios',
      '#title' => $this->t('Include hover navigation Javascript'),
      '#description' =>
      $this->t('This will also open submenus when hovering over their parent menu item.'),
      '#default_value' => $this->configuration['include_hover_js'],
      '#options' => [
        1 => 'Yes',
        0 => 'No',
      ],
      '#states' => [
        'visible' => [
          ':input[name="settings[include_default_js]"]' => ['value' => 1],
        ],
      ],
    ];

    $form['hover_show_delay'] = [
      '#type' => 'number',
      '#min' => 0,
      '#title' => $this->t('Hover show delay'),
      '#description' =>
      $this->t('How many milliseconds to wait before displaying the submenu when hovering over the parent menu item'),
      '#default_value' => $this->configuration['hover_show_delay'],
      '#states' => [
        'visible' => [
          ':input[name="settings[include_default_js]"]' => ['value' => 1],
          ':input[name="settings[include_hover_js]"]' => ['value' => 1],
        ],
      ],
    ];

    $form['hover_hide_delay'] = [
      '#type' => 'number',
      '#min' => 0,
      '#title' => $this->t('Hover hide delay'),
      '#description' =>
      $this->t('How many milliseconds to wait before hiding the submenu when the pointer is no longer hovering over the parent menu link or the submenu'),
      '#default_value' => $this->configuration['hover_hide_delay'],
      '#states' => [
        'visible' => [
          ':input[name="settings[include_default_js]"]' => ['value' => 1],
          ':input[name="settings[include_hover_js]"]' => ['value' => 1],
        ],
      ],
    ];

    return $form;
  }

  /**
   * Overrides \Drupal\block\BlockBase::blockSubmit().
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $this->configuration['submenu_disclosure_levels'] = $form_state->getValue('submenu_disclosure_levels');
    $this->configuration['include_default_js'] = $form_state->getValue('include_default_js');
    $this->configuration['include_hover_js'] = $form_state->getValue('include_hover_js');
    $this->configuration['hover_show_delay'] = $form_state->getValue('hover_show_delay');
    $this->configuration['hover_hide_delay'] = $form_state->getValue('hover_hide_delay');
  }

  /**
   * Implements \Drupal\block\BlockBase::build().
   */
  public function build() {
    $build = parent::build();
    $build['#theme'] = 'menu__disclosure';

    if ($this->configuration['include_default_js']) {
      $build['#attached']['library'][] = 'disclosure_menu/menu';
      if ($this->configuration['include_hover_js']) {
        $build['#attached']['library'][] = 'disclosure_menu/hover';
        $build['#attached']['drupalSettings']['disclosureMenu']['hoverShowDelay'] = $this->configuration['hover_show_delay'];
        $build['#attached']['drupalSettings']['disclosureMenu']['hoverHideDelay'] = $this->configuration['hover_hide_delay'];
      }
    }

    $build['#submenu_disclosure_levels'] = $this->configuration['submenu_disclosure_levels'];

    return $build;
  }

}
